/* $Workfile: A16.c $       $Revision: 16 $      $Modtime: 5/18/01 10:32a $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc PublicDemo

  @module

    A16.c

    A simple example console application demonstrating the use of OTX Driver
    with the ARNI-16 POTS board.

    It configures 2 out of the 16 POTS interfaces.  It also creates a loopback in the codecs
    with HS0 connected to HS1 and HS2 connected to HS3.

    The sample application provides a simple console interface which utilizes
    simple keystrokes as commands.

    Press 'h' or '?' for help. 'q' to quit.

    Press '+' or '-' to select another LI as the active LI.
    Press 'f' to go off-hook. press 'n' to go on-hook.
    Press 'p' or 'P' to pulse dial a number.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <assert.h>
#include <time.h>


#include "OsaStd.h"
#include "OsaCon.h"
#include "OsaThrd.h"
#include "OtxErr.h"
#include "OtxType.h"
#include "OtxBrdD.h"
#include "OtxDrv.h"
#include "OtxPots.h"

/*******************************************************************************
 Macros, Types
*******************************************************************************/
#define DEMO_TASK_REF          1L          // Task Reference for a non-blocking call
#define DEMO_POTS_LI_MAX        2          // Number of POTS lines supported by the Demo
#define DEMO_POTS_CTRL_MAX      1          // Use only the first LI Controller. Arni-16 has 2
                                           // LI controllers each supporting 8 LIs
#define DEMO_CODEC_MAX          4          // Arni Board contains 4 codecs. Use 4 in this demo

#define DEMO_DEFAULT_LOG_FILE "A16.log"  // Default name of the LOG file

typedef struct DemoEnvT {
  char           *logFileName;
  OtxPotsCfgSetE  nSetId;
} DemoEnvT;


/*******************************************************************************
 Function Prototypes
*******************************************************************************/
static void DemoCreateEventQueue(void);
static void DemoOpenPhysicalDevices(void);
static void DemoCreateLogicalDevices(void);
static void DemoConnectDevices(void);
static void DemoInitializeLogicalDevices(void);
static void DemoEnableDevices(void);
static void DemoPrintEvent(IN OtxEventDataS *pEventData);
static void DemoCheckRc(IN OTX_RESULT otxRc);
static void DemoLogf(char *formatStr, ...);
static void DemoHelpPanel(void);
static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP);


/*******************************************************************************
 Global Data
*******************************************************************************/
OTX_EVENT   g_hNotificationEvent = OTX_INVALID_EVENT_VALUE;
OTX_HANDLE  g_hEventQueue        = OTX_INVALID_HANDLE_VALUE;
FILE       *g_pDemoLogFile = NULL;
DemoEnvT    g_demoEnv;

struct {
  OTX_HANDLE hBoardController0;
  OTX_HANDLE hLiController[DEMO_POTS_CTRL_MAX];
  OTX_HANDLE hLi[DEMO_POTS_LI_MAX];
  OTX_HANDLE hCodec[DEMO_CODEC_MAX];
} g_hPhysDevs;

struct {
  OTX_HANDLE hCodecToCodec[DEMO_CODEC_MAX];
} g_hPipes;

char *g_sqNumber0 = "102";

/*******************************************************************************
*******************************************************************************/


void DemoCreateEventQueue(void)
/*******************************************************************************
  @func
    Create the event queue to be used by the driver to inform the application
    about events, such as line status changes, or detected DMTF digits.
*******************************************************************************/
{
  OTX_RESULT result;

  // Create a notification Event
  result = OtxDrvCreateEvent(&g_hNotificationEvent); DemoCheckRc(result);

  // Create Event Queue for driver notifications
  result = OtxDrvCreateEventQueue(128, g_hNotificationEvent, &g_hEventQueue); DemoCheckRc(result);

}



void DemoOpenPhysicalDevices(void)
/*******************************************************************************

  @func
    Opens the Physical Devices on the Arni-32 board

  @comm
    Calls function <f OtxDrvOpenPhysicalDevice> returning a handle to a
    Device Object representing a physical device, which can be the
    entire board or an integrated circuit (chip) on the board. The number of
    physical devices is set per board and cannot change dynamically.

*******************************************************************************/
{
  OTX_RESULT result;
  OTX_INT32  j;
  OTX_INT32  i;
  OTX_INT32 nLi = 0;
  OTX_INT32 nLiMax = 8;   // Max 8 POTS Li's per LI Controller

  // Get Handle to the Physical ARNI Board Controller
  DemoLogf("Open Board OTX_DEVICE_ARNI_PCI\n");
  result = OtxDrvOpenPhysicalDevice(0, OTX_DEVICE_ARNI_PCI, 0, 0, g_hEventQueue, &g_hPhysDevs.hBoardController0); DemoCheckRc(result);

  if (DEMO_POTS_LI_MAX < nLiMax) {
    nLiMax = DEMO_POTS_LI_MAX;
  }

  // Connect to the POTS LI (0-7) Controller
  for (j = 0; j < DEMO_POTS_CTRL_MAX; j++) {
    DemoLogf("Open OTX_DEVICE_LI_CONTROLLER  #%d (AIC)\n", j);
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hBoardController0, OTX_DEVICE_LI_CONTROLLER, j, j, g_hEventQueue, &g_hPhysDevs.hLiController[j]); DemoCheckRc(result);

    DemoLogf("Configuring AIC#%d for country set=");
    switch (g_demoEnv.nSetId) {
    case OTX_POTS_CFG_SET_USA:
      DemoLogf("USA");
      break;
    case OTX_POTS_CFG_SET_GERMANY: 
      DemoLogf("Germany");
      break;
    case OTX_POTS_CFG_SET_UK:
      DemoLogf("United Kingdom");
      break;
    case OTX_POTS_CFG_SET_FRANCE:
      DemoLogf("France");
      break;
    case OTX_POTS_CFG_SET_AUSTRALIA:
      DemoLogf("Australia");
      break;
    case OTX_POTS_CFG_SET_JAPAN:
      DemoLogf("Japan");
      break;
    default:
      DemoLogf("UNKNOWN");
      break;
    }
    DemoLogf("\n");
  
    result = OtxDrvSetAttributeValueUint32(g_hPhysDevs.hLiController[j], OTX_ATTR_POTS_CFG_SET, (OTX_UINT32)g_demoEnv.nSetId); DemoCheckRc(result);

    // Get handles to POTS line devices
    for (i = 0; i < nLiMax; i++) {
      DemoLogf("Open OTX_DEVICE_LI_POTS  #%d \n", i);
      result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hLiController[j], OTX_DEVICE_LI_POTS, i, j*nLiMax + i, g_hEventQueue, &g_hPhysDevs.hLi[nLi]); DemoCheckRc(result);
      nLi++;
    }
  }


  // Get a handle to the Codec Devices
  for (i = 0; i < DEMO_CODEC_MAX; i++) {
    DemoLogf("Open OTX_DEVICE_CODEC\n");
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hBoardController0, OTX_DEVICE_CODEC, i, i, g_hEventQueue, &g_hPhysDevs.hCodec[i]); DemoCheckRc(result);
  }

}



void DemoCreateLogicalDevices(
/*******************************************************************************

  @func
    Create the needed Logical Devices

  @comm
    The logical devices are differenciated from the phyiscal devices by the fact
    the number of logical devices is not set. More of them can be requested
    by the user and allocated by the system if needed.

*******************************************************************************/
  void
)
{
  // None in this demo program
}


void DemoConnectDevices(
/*******************************************************************************

  @func
    Create the needed Pipe Devices and connect the physical and logical devices
    to pipes.

  @comm
    A Pipe Device is a logical entity representing a serial bit-stream (a subset of a
    Highway) with a data rate between 8 kbit/s and 2.048Mbit/s. A pipe is a characterized
    bu its capacity (in 8 kbit/s increments). The user can create a pipe with
    the driver function <f OtxDrvCreatePipe>. A pipe is connected to physical or
    logical devices and mapped to specific time-slots with the driver function
    <f OtxDrvConnectPipe>.

*******************************************************************************/
  void
)
{
  OTX_RESULT result;
  OTX_INT32  i;

  DemoLogf("In DemoConnectDevices\n");

  for (i = 0; i < DEMO_CODEC_MAX; i++) {
    result = OtxDrvCreatePipe(64, &g_hPipes.hCodecToCodec[i]); DemoCheckRc(result);
  }


  // Connect Codec 0 to Codec 1
  result = OtxDrvConnectPipe(g_hPipes.hCodecToCodec[0], OTX_PIPE_OUTPUT, g_hPhysDevs.hCodec[0], -1, NULL, 0); DemoCheckRc(result);
  result = OtxDrvConnectPipe(g_hPipes.hCodecToCodec[0], OTX_PIPE_INPUT , g_hPhysDevs.hCodec[1], -1, NULL, 0); DemoCheckRc(result);
  result = OtxDrvConnectPipe(g_hPipes.hCodecToCodec[1], OTX_PIPE_OUTPUT, g_hPhysDevs.hCodec[1], -1, NULL, 0); DemoCheckRc(result);
  result = OtxDrvConnectPipe(g_hPipes.hCodecToCodec[1], OTX_PIPE_INPUT , g_hPhysDevs.hCodec[0], -1, NULL, 0); DemoCheckRc(result);
  
  // Connect Codec 2 to Codec 3
  result = OtxDrvConnectPipe(g_hPipes.hCodecToCodec[2], OTX_PIPE_OUTPUT, g_hPhysDevs.hCodec[2], -1, NULL, 0); DemoCheckRc(result);
  result = OtxDrvConnectPipe(g_hPipes.hCodecToCodec[2], OTX_PIPE_INPUT , g_hPhysDevs.hCodec[3], -1, NULL, 0); DemoCheckRc(result);
  result = OtxDrvConnectPipe(g_hPipes.hCodecToCodec[3], OTX_PIPE_OUTPUT, g_hPhysDevs.hCodec[3], -1, NULL, 0); DemoCheckRc(result);
  result = OtxDrvConnectPipe(g_hPipes.hCodecToCodec[3], OTX_PIPE_INPUT , g_hPhysDevs.hCodec[2], -1, NULL, 0); DemoCheckRc(result);

}


void DemoInitializeLogicalDevices(
/*******************************************************************************
  @func
    Initialize the logical device. This function must be called after
    the devices are created and connected.
*******************************************************************************/
  void
)
{
  // Nothing in this demo program
}




void DemoEnableDevices(void)
/*******************************************************************************
  @func
    Enable Configured Devices
*******************************************************************************/
{
  OTX_RESULT result;
  OTX_INT32  i;

  DemoLogf("In DemoEnableDevices\n");
  // Enable Physical Devices
  result = OtxDrvEnable(g_hPhysDevs.hBoardController0,     OTX_TASK_SYNC, 5000L); DemoCheckRc(result);

  for (i = 0; i < DEMO_POTS_CTRL_MAX; i++) {
    result = OtxDrvEnable(g_hPhysDevs.hLiController[i],   OTX_TASK_SYNC, 5000L); DemoCheckRc(result);
  }

  for (i = 0; i < DEMO_POTS_LI_MAX; i++) {
    result = OtxDrvEnable(g_hPhysDevs.hLi[i],             OTX_TASK_SYNC, 5000L); DemoCheckRc(result);
  }

  for (i = 0; i < DEMO_CODEC_MAX; i++) {
    result = OtxDrvEnable(g_hPhysDevs.hCodec[i],          OTX_TASK_SYNC, 5000L); DemoCheckRc(result);
  }


}


static void DemoCheckRc(IN OTX_RESULT result)
/*******************************************************************************
  @func
    Check the result from API call. If the result is a failure code then prompt
    the user if he wants to exit.
*******************************************************************************/
{
  short c;
  if ( OTX_FAILURE(result) ) {
    DemoLogf("Error: rc = %X\n", result);

    DemoLogf("Do you want to exit (Y/N) ");
    c = getch();
    if (c == 'y' || c == 'Y') {
      // Delete allocated resources. IMPORTANT TO CALL BEFORE APPLICATION EXIT!! (for now, but not in future releases of the driver)
      OtxDrvReleaseLib();
			OsaConsoleExit();
      exit(1);
    }

  }
}


/*******************************************************************************
  @func
    Prints a timestamp in a MM-DD hh:mm:ss:mmm format, where

    MM  = month
    DD  = day (of month)
    hh  = hour of day
    mm  = minutes of day
    ss  = seconds of day
    mmm = milliseconds

*******************************************************************************/
void DemoPrintTimeStamp(
  OTX_DATETIME nTimeStamp
)
{
  OTX_DATETIME nTimeStampMs;
  OTX_DATETIME nTimeStampSec;
  struct tm *ptCurrentDayAndTime;
  time_t nCurrentTimeSecs;
  
  // Subtract number of 100ns ticks between Jan 1, 1970 and Jan 1, 1600
  
  #if defined(__linux__)
    nTimeStamp.nUint64 = nTimeStamp.nUint64 - 1164447360 * 100000000;  
  #elif defined(WIN32)
    nTimeStamp.nUint64 = nTimeStamp.nUint64 - 116444736000000000;  
  #endif

  
  // Convert to seconds since Jan 1, 1970 (so we can use this time with localtime())
  nTimeStampSec.nUint64 = nTimeStamp.nUint64 / 10000000;
  
  // Calculate number of milliseconds (since number of seconds is not a very accurate way to report timestamp)
  nTimeStampMs.nUint64 = nTimeStamp.nUint64 / 10000 - (nTimeStampSec.nUint64 * 1000);
  
  nCurrentTimeSecs = nTimeStampSec.sUint32.nLow;
  ptCurrentDayAndTime = localtime(&nCurrentTimeSecs);
  
  if (ptCurrentDayAndTime) {
    
    DemoLogf("%.2d-%.2d %.2d:%.2d:%.2d.%.3d ", 
      ptCurrentDayAndTime->tm_mon + 1,
      ptCurrentDayAndTime->tm_mday,
      ptCurrentDayAndTime->tm_hour,
      ptCurrentDayAndTime->tm_min, 
      ptCurrentDayAndTime->tm_sec,
      nTimeStampMs.sUint32.nLow
      );
  } else {
    DemoLogf("Error: Invalid timestamp (before Jan 1, 1970)\n");
  }
    
}


/*******************************************************************************
  @func
    Idle Function that check for Events from the OTX Driver
*******************************************************************************/
void  *DemoIdleFunction(
  void *lpParameter   // thread data
)
{
  OtxEventDataS eventData;
  OTX_RESULT result;

  while (1) {
    OtxDrvWaitForSingleEvent(g_hNotificationEvent, OTX_TIME_INFINITY);
    //DemoLogf("Event signalled\n");
    do {
      result = OtxDrvGetEventData(g_hEventQueue, &eventData); DemoCheckRc(result);

      if (result == OTX_S_OK) {
        // We have a driver event
        // Printf the event data.
        // In a real application branch based on the data
        DemoPrintEvent(&eventData);

      }
    } while (result == OTX_S_OK);
  }

  return 0;

}


void DemoPrintEvent(
/*******************************************************************************
  @func
    Prints Driver Notification Events
*******************************************************************************/
  OtxEventDataS *pEventData
)
{

  DemoPrintTimeStamp(pEventData->m_nTimeStamp);

  switch (pEventData->m_eDeviceType) {

  case OTX_DEVICE_LI_POTS:
    // Event from POTS Line Interface Device
    switch (pEventData->m_nCode) {
    case OTX_POTS_RING_SIGNAL_DETECTED:
      DemoLogf("Li[%d] ring signal detected\n", pEventData->m_nSourceId);
      break;
    case OTX_POTS_RING_SIGNAL_ON:
      DemoLogf("Li[%d] ring pulse on\n", pEventData->m_nSourceId);
      break;
    case OTX_POTS_RING_SIGNAL_OFF:
      DemoLogf("Li[%d] ring pulse off\n", pEventData->m_nSourceId);
      break;
    case OTX_POTS_INVALID_RING_SIGNAL:
      DemoLogf("Li[%d] invalid ring signal\n", pEventData->m_nSourceId);
      break;
    case OTX_POTS_CALLER_ID:
      DemoLogf("Li[%d] Caller ID received\n", pEventData->m_nSourceId);
      break;
    default:
      // Unexpected event from this device
      DemoLogf("Li[%d] Unexpected Event\n", pEventData->m_nSourceId);
    }
    break;

  default:
    // Unhandled event
    DemoLogf("Event from an unknown object 0x%X\n", pEventData->m_hDevice);
    break;
  }

}



OTX_INT32 DemoProcessUserCommand(char commandChar)
/*******************************************************************************
  @func
    Processes User Commands
*******************************************************************************/
{
  OTX_RESULT        result;
  static OTX_INT32  nCurrentPotsLine = 0;
  static OTX_HANDLE hCurrentPotsLine;
  hCurrentPotsLine = g_hPhysDevs.hLi[nCurrentPotsLine];

  switch (commandChar) {

  case '?':
  case 'h':
  case 'H':
    DemoLogf("Cmd  Synopsis\n");
    DemoLogf("===  ========\n");
    DemoLogf(" ?    Display Help text\n");
    DemoLogf(" h    Ditto\n");
    DemoLogf(" q    Quit\n");
    DemoLogf(" +    Select the next POTS line as active POTS Line (target of subsequent commands)\n");
    DemoLogf(" -    Select the previous POTS line as the active POTS Line (target of subsequent commands)\n");
    DemoLogf(" f    Set active POTS Line Off-Hook\n");
    DemoLogf(" n    Set active POTS Line On-Hook\n");
    DemoLogf(" g    Get current Time\n");
    break;

  case 'g':
    {
      OTX_DATETIME nTimeStamp;

      result = OtxDrvGetCurrentTime(&nTimeStamp); DemoCheckRc(result);

      DemoLogf("Current time is ");
      DemoPrintTimeStamp(nTimeStamp);
      DemoLogf("\n");
    }
    break;


  case '-':
    {                           
      // Select the previous Line Interface (LI) as the Active one
      //OTX_INT32 nPrevPotsLine = nCurrentPotsLine;

      if ( nCurrentPotsLine == 0 ) {
        nCurrentPotsLine = DEMO_POTS_LI_MAX - 1;
      } else {
        nCurrentPotsLine--;
      }

      hCurrentPotsLine = g_hPhysDevs.hLi[nCurrentPotsLine];
      DemoLogf("POTS Line # %d Selected as the Active Line\n", nCurrentPotsLine);
    }
    break;

  case '+':
    {
      // Select the next Line Interface (LI) as the Active one
      //OTX_INT32 nPrevPotsLine = nCurrentPotsLine;

      if ( nCurrentPotsLine == (DEMO_POTS_LI_MAX - 1) ) {
        nCurrentPotsLine = 0;
      } else {
        nCurrentPotsLine++;
      }

      hCurrentPotsLine = g_hPhysDevs.hLi[nCurrentPotsLine];
      DemoLogf("POTS Line # %d Selected as the Active Line\n", nCurrentPotsLine);
    }
    break;

  case 'f':
    // Set Active POTS Line Off-Hook
    DemoLogf("POTS Line #%d Off-hook\n", nCurrentPotsLine);
    result = OtxPotsOffHook(hCurrentPotsLine); DemoCheckRc(result);
    DemoLogf("Done.\n");
    break;

  case 'n':
    // Set Active POTS Line On-Hook
    DemoLogf("POTS Line #%d On-hook\n", nCurrentPotsLine);
    result = OtxPotsOnHook(hCurrentPotsLine); DemoCheckRc(result);
    DemoLogf("Done.\n");
    break;

  case 'p':
    // Pulse dial g_sqNumber0
    DemoLogf("POTS Line #%d Pulse Dialing '%s'\n", nCurrentPotsLine, g_sqNumber0);
    result = OtxPotsPulseDial(hCurrentPotsLine, 40, 60, 500, (OTX_UINT8 *) g_sqNumber0, strlen(g_sqNumber0), DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(result);
    DemoLogf("Done.\n");
    break;

  case 'm':
    {
      // Read the operation mode of the Active LI Device ...
      OtxPotsLiModeE eLiMode;

      result = OtxPotsGetLiMode(hCurrentPotsLine, &eLiMode);
      DemoLogf("POTS Line #%d is in mode 0x%.4X\n", nCurrentPotsLine, eLiMode);
    }
    break;

  case 'M':
    {
      // Set the operation mode of the Active LI Device ...
      OtxPotsLiModeE eLiMode;
      char wValStr[32];

      DemoLogf("Set POTS Line #%d Mode\n");
      DemoLogf("Sleep         :  0\n");
      DemoLogf("Ringing       :  1\n");
      DemoLogf("Conversation  :  2\n");
      DemoLogf("Pulse Dialing :  3\n");
      OsaConsoleGetStr(wValStr);
      eLiMode = (OtxPotsLiModeE) strtoul(wValStr, NULL, 0);


      result = OtxPotsSetLiMode(hCurrentPotsLine, eLiMode);
      DemoLogf("POTS Line #%d is set to mode %d\n", nCurrentPotsLine, eLiMode);
    }
    break;

  case 'q':
  case 'Q':
    // Quit ...
    return 1;
    break;

  default:
    DemoLogf("Unknown command. Press '?' for help, 'q' to quit\n");
    break;
  }

  return 0;
}



static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP) {
/*******************************************************************************
  @func
    Parse the program's command line arguments and calls upon
    requested actions.

    There are some optional switches that can be used following
    the driver name on the command line. Switches are are
    separated by spaces, and can appear in arbitrary order.
    Switches consist of a hyphen and a letter:
*******************************************************************************/

  short  i;

  // Set reasonable defaults before reading from command line
  envP->logFileName  = DEMO_DEFAULT_LOG_FILE;
  envP->nSetId       = OTX_POTS_CFG_SET_USA;

  for (i = 1; i < argc; i++) {        // Scan command line arguments
    OsaStdStrUpr(argv[i]);
    if (*argv[i] == '-' || *argv[i] == '/') {  // Is argument a legal switch?
      if (*(argv[i]+1) == 'L') {   // Log file
        envP->logFileName = argv[i]+2;
      } else if (*(argv[i]+1) == 'S') {   // Configuration Set
        if (*(argv[i]+2) == 'U') {          // USA
          envP->nSetId = OTX_POTS_CFG_SET_USA;
        } else if (*(argv[i]+2) == 'G') {   // Germany
          envP->nSetId = OTX_POTS_CFG_SET_GERMANY;
        } else if (*(argv[i]+2) == 'E') {   // UK
          envP->nSetId = OTX_POTS_CFG_SET_UK;
        } else if (*(argv[i]+2) == 'F') {   // France
          envP->nSetId = OTX_POTS_CFG_SET_FRANCE;
        } else if (*(argv[i]+2) == 'A') {   // Australia
          envP->nSetId = OTX_POTS_CFG_SET_AUSTRALIA;
        } else if (*(argv[i]+2) == 'J') {   // Japan
          envP->nSetId = OTX_POTS_CFG_SET_JAPAN;
        } else {
          DemoLogf("Error: Unsupported country specific configuration set %s\n", argv[i]);
          DemoHelpPanel();
					OsaConsoleExit();
          exit(1);
        }
      } else {
        DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
        DemoHelpPanel();
				OsaConsoleExit();
        exit(1);
      }
    } else {
      DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
      DemoHelpPanel();
			OsaConsoleExit();
      exit(1);
    }
  }

}


static void DemoHelpPanel()
/*******************************************************************************
  @func
    Displays a help for the command line.
*******************************************************************************/
{
  DemoLogf("\nCommand line switches:\n");
  DemoLogf(" /L<fileName> : Log file name\n");
  DemoLogf(" /SU          : Li Configuration for USA (Default)\n");
  DemoLogf(" /SG          : Li Configuration for Germany\n");
  DemoLogf(" /SE          : Li Configuration for United Kingdom\n");
  DemoLogf(" /SF          : Li Configuration for France\n");
  DemoLogf(" /SA          : Li Configuration for Australia\n");
  DemoLogf(" /SJ          : Li Configuration for Japan\n");
}


static void DemoLogf(char *formatStr, ...)
/*******************************************************************************
  @func
    Displays a message on the screen and prints it to the logfile.
*******************************************************************************/
{
  va_list argList;
  va_start(argList, formatStr);

  OsaConsolePrintF(formatStr, argList);
  if (g_pDemoLogFile) {
    vfprintf(g_pDemoLogFile,   formatStr, argList);
  }

  va_end(argList);
}



int main(int argc, char *argv[])
/*******************************************************************************
*******************************************************************************/
{
  OTX_INT32  exitFlag = 0;
  OTX_RESULT result;  // OTX Standard Return Code
  char       commandChar;

	OsaConsoleInit();

  DemoLogf("Arni-16-PCI Demo Program\n");

  DemoParseCmdLineArgs(argc, argv, &g_demoEnv);

  if ((g_pDemoLogFile = fopen(g_demoEnv.logFileName, "wt")) == NULL) {
    printf("Couldn't open logfile %s\n", g_demoEnv.logFileName);
    OsaConsoleExit();
		exit(-1);
  }

  DemoLogf("%s, %s, Odin TeleSystems Inc.\n", argv[0], __DATE__);
  DemoLogf("Log file is %s\n", g_demoEnv.logFileName);


  // Query Library revision
  {
    char szVersionInfo[255];
    OTX_BOOL bIsStaticLinkedLibrary;
    char szLibFileName[64];
    result = OtxDrvGetAttributeValueSz(OTX_LIB_HANDLE, OTX_ATTR_LIB_REV_STR, szVersionInfo, 255); DemoCheckRc(result);
    result = OtxDrvGetAttributeValueBool(OTX_LIB_HANDLE, OTX_ATTR_LIB_TYPE_STATIC, &bIsStaticLinkedLibrary); DemoCheckRc(result);
    result = OtxDrvGetAttributeValueSz(OTX_LIB_HANDLE, OTX_ATTR_LIB_FILE_NAME, szLibFileName, 64); DemoCheckRc(result);
    
    DemoLogf("Library version %s (filename %s, %s linked)\n", szVersionInfo, szLibFileName, bIsStaticLinkedLibrary ? "static":"dynamic");
  }


  // Connect to the Driver
  DemoLogf("Connecting lib...\n");
  result = OtxDrvConnectLib(); 
  switch (OTX_RESULT_CODE(result)) {
  case OTX_RES_HANDLE:
    DemoLogf("Error: Could not connect to the OTX library.\nPossible cause: OTX Driver not loaded\n");
    OsaConsoleExit();
    OtxDrvReleaseLib();
    exit(1);
    break;
  case OTX_RES_NOT_SUPPORTED:
    DemoLogf("Error: Mismatch between OTX Driver revision and OTX Library revision.\nRevisions must match.\n");
    OsaConsoleExit();
    OtxDrvReleaseLib();
    exit(1);
    break;
  default:
    DemoCheckRc(result);
    break;
  }

  // Create Event Queus for Driver --> Application Communications
  DemoCreateEventQueue();

  // Connect (get handles) to the physical devices on the Arni-32 board
  DemoOpenPhysicalDevices();

  // Create and get handles to the Logical devices needed by this application
  DemoCreateLogicalDevices();

  // Create and get handles to the Pipe devices needed by this application
  // Connect Pipes to Physical and Logical Devices
  DemoConnectDevices();

  // Initialized the Connected Logical Devices
  DemoInitializeLogicalDevices();

  // Enable all devices. Their device state should now be OTX_DEVICE_WORKING
  DemoEnableDevices();

  // Read Serial Number from the board
  {
    OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
    OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
    OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
    char *pszSerialNumber;

    result = OtxDrvGetAttributeType(g_hPhysDevs.hBoardController0, OTX_ATTR_BRD_SERIAL_NUMBER, &eAttribType, &cbAttribValue, &bReadOnly); DemoCheckRc(result);
    assert(eAttribType == OTX_ATTRIB_SZ);
    assert(bReadOnly == OTX_TRUE);
    pszSerialNumber = (char *) malloc(cbAttribValue * sizeof(char));
    result = OtxDrvGetAttributeValueSz(g_hPhysDevs.hBoardController0, OTX_ATTR_BRD_SERIAL_NUMBER, pszSerialNumber, cbAttribValue); DemoCheckRc(result);
    DemoLogf("Board Serial Number: %s\n", pszSerialNumber);
    free(pszSerialNumber);
  }

  // Read Build Number from the board
  {
    OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
    OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
    OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
    char *pszBrdBuildNumber;

    result = OtxDrvGetAttributeType(g_hPhysDevs.hBoardController0, OTX_ATTR_BRD_BUILD, &eAttribType, &cbAttribValue, &bReadOnly); DemoCheckRc(result);
    assert(eAttribType == OTX_ATTRIB_SZ);
    assert(bReadOnly == OTX_TRUE);
    pszBrdBuildNumber = (char *) malloc(cbAttribValue * sizeof(char));
    result = OtxDrvGetAttributeValueSz(g_hPhysDevs.hBoardController0, OTX_ATTR_BRD_BUILD, pszBrdBuildNumber, cbAttribValue); DemoCheckRc(result);
    DemoLogf("Board Build Number: %s\n", pszBrdBuildNumber);
    free(pszBrdBuildNumber);
  }

  // Read CPLD Build Numbers from the board
  {
    OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
    OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
    OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
    char *pszCpldBuildNumbers;

    result = OtxDrvGetAttributeType(g_hPhysDevs.hBoardController0, OTX_ATTR_BRD_CPLD_BUILD, &eAttribType, &cbAttribValue, &bReadOnly); DemoCheckRc(result);
    assert(eAttribType == OTX_ATTRIB_SZ);
    assert(bReadOnly == OTX_TRUE);
    pszCpldBuildNumbers = (char *) malloc(cbAttribValue * sizeof(char));
    result = OtxDrvGetAttributeValueSz(g_hPhysDevs.hBoardController0, OTX_ATTR_BRD_CPLD_BUILD, pszCpldBuildNumbers, cbAttribValue); DemoCheckRc(result);
    DemoLogf("CPLD Build:\n%s", pszCpldBuildNumbers);
    free(pszCpldBuildNumbers);
  }


  // Main Loop. Loop here and wait for Events from the driver or commands from the user ....

  OsaThreadStart(DemoIdleFunction);

  DemoLogf("\nPress 'h' for help, 'q' to quit ...\n");

  while (!exitFlag) {
    commandChar = getch();
    exitFlag = DemoProcessUserCommand(commandChar);
  }

  // Delete allocated resources.
  OtxDrvReleaseLib();
	OsaConsoleExit();

  return 0;
}

