#ifndef OTX_T1E1_H
#define OTX_T1E1_H
/*******************************************************************************

  Copyright (C) 1998, 1999 Odin TeleSystems Inc., All Rights Reserved

  @doc PublicHwApi OtxT1E1

  OTX Generic Hardware Driver API

  @module
  OtxT1E1 | This file contains function prototypes for the T1/E1 Line Interface devices

  @contents2
  OtxT1E1.h |

  @index func | OtxT1E1

  @contents2
  OTX T1/E1 Line Interface Device Functions - OtxT1XXX(), OtxE1XXX() |
  The user functions available for the T1/E1 Line Interface Transceiver Devices

  @head3 OtxT1E1XXX(), OtxT1XXX(), and OtxE1XXX() Function Reference |

  @index func | OtxT1E1

*******************************************************************************/

#include "OtxDef.h"

#include "OtxT1E1D.h"

#ifdef __cplusplus
  extern "C" {
#endif


/*******************************************************************************

  @func

  Sets up the T1/E1 Line Interface transceiver in either T1 or E1 mode.

  @xref
  <t OtxT1E1LiModeE>

*******************************************************************************/
OTX_RESULT OtxT1E1SetLiMode(
  IN OTX_HANDLE hLineDevice,     // @parm Handle to the T1/E1 Transceiver Device
  IN OtxT1E1LiModeE eT1E1Mode    // @parm T1 or E1 mode
);


/*******************************************************************************

  @func

  Configures the T1 Line Interface transceiver.

  @xref
  <f OtxT1GetConfiguration>
  <nl><f OtxE1SetConfiguration>
  <nl><t OtxT1ConfigOptionsS>

*******************************************************************************/
OTX_RESULT OtxT1SetConfiguration(
  IN OTX_HANDLE hLineDevice,                // @parm Handle to the T1/E1 Transceiver Device
  IN OtxT1ConfigOptionsS *pT1ConfigOptions  // @parm T1 Configuration Options
);


/*******************************************************************************

  @func
    Retrieves the current T1 Line Interface configuration data.

  @xref
  <f OtxT1SetConfiguration>
  <nl><f OtxE1SetConfiguration>
  <nl><t OtxT1ConfigOptionsS>

*******************************************************************************/
OTX_RESULT OtxT1GetConfiguration(
  IN  OTX_HANDLE hLineDevice,                // @parm Handle to the T1/E1 Transceiver Device
  OUT OtxT1ConfigOptionsS *pT1ConfigOptions  // @parm T1 Configuration Data
);



/*******************************************************************************

  @func
    Configures the E1 Line Interface transceiver.

  @xref
    <f OtxE1GetConfiguration>
    <nl><f OtxT1SetConfiguration>
    <nl><t OtxE1ConfigOptionsS>

*******************************************************************************/
OTX_RESULT OtxE1SetConfiguration(
  IN OTX_HANDLE hLineDevice,                // @parm Handle to the T1/E1 Transceiver Device
  IN OtxE1ConfigOptionsS *pE1ConfigOptions  // @parm E1 Configuration Data
);


/*******************************************************************************

  @func

  Retrieves the current E1 Line Interface configuration data.

  @xref
    <f OtxE1SetConfiguration>
    <nl><f OtxT1SetConfiguration>
    <nl><t OtxE1ConfigOptionsS>

*******************************************************************************/
OTX_RESULT OtxE1GetConfiguration(
  IN  OTX_HANDLE hLineDevice,                // @parm Handle to the T1/E1 Transceiver Device
  OUT OtxE1ConfigOptionsS *pE1ConfigOptions  // @parm E1 Configuration Data
);



/*******************************************************************************

  @func
    Initiates a sending of an alarm or a series of alarms on the
    T1/E1 physical layer (depending on alarmType)

  @xref
    <t OtxT1E1AlarmTypeE>

*******************************************************************************/
OTX_RESULT OtxT1E1AlarmOn(
  IN OTX_HANDLE hLineDevice,           // @parm Handle to the T1/E1 Transceiver Device
  IN OtxT1E1AlarmTypeE eAlarmType      // @parm Alarm type to be sent to the Remote End
);



/*******************************************************************************

  @func

  Clears an earlier initiated alarm.

  @xref
  <t OtxT1E1AlarmTypeE>

*******************************************************************************/
OTX_RESULT OtxT1E1AlarmOff(
  IN OTX_HANDLE hLineDevice,           // @parm Handle to the T1/E1 Transceiver Device
  IN OtxT1E1AlarmTypeE eAlarmType      // @parm Alarm type to turned off
);



/*******************************************************************************

  @func
    Loops the Line Interface. I.e. received E1/T1 data will be transmitted back
    on the transmit pairs.

  @xref
    <t OtxT1E1LoopE>

*******************************************************************************/
OTX_RESULT OtxT1E1SetLineLoop(
  IN OTX_HANDLE hLineDevice,   // @parm Handle to the T1/E1 Transceiver Device
  OtxT1E1LoopE eLoopType       // @parm Loop Type: Line loop or Remote loop
);



/*******************************************************************************

  @func
    Initiates a resynchronization procedure of the pulse frame and the
    CRC-multiframe starting directly after the old framing candidate.

*******************************************************************************/
OTX_RESULT OtxT1E1ForceResynch(
  IN OTX_HANDLE hLineDevice       // @parm Handle to the T1/E1 Transceiver Device
);


/*******************************************************************************

  @func
    Read the physical line status of the T1/E1 Line Interface.

*******************************************************************************/
OTX_RESULT OtxT1E1GetLayer1Status(
  IN OTX_HANDLE hLineDevice,               // @parm Handle to the T1/E1 Transceiver Device
  OUT OtxT1E1Layer1StatusE *eLayer1Status  // @parm T1/E1 Physical Layer Line Status. Either <e OtxT1E1ResultCodesE.OTX_T1E1_RES_L1_DOWN> or <e OtxT1E1ResultCodesE.OTX_T1E1_RES_L1_OK>
);


/*******************************************************************************

  @func
    Enable or Disable (tri-state) the Transmit pair of the Line Interface.

*******************************************************************************/
OTX_RESULT OtxT1E1XmitPinControl(
  IN OTX_HANDLE hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN OTX_BOOL   bTxPinEnable      // @parm OTX_TRUE enable, OTX_FALSE disables (tri-state) the Line Interface Transmit Pins
);

/*******************************************************************************

  @func
    Sets the line termination for the E1/T1 span of the Line Interface.

*******************************************************************************/
OTX_RESULT OtxT1E1SetLineTermination(
  IN OTX_HANDLE hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN OtxT1E1TermModeE eTermMode   // @parm Termination mode to match impedance of the cable connected to the Line Interface
);


#ifdef  _TURNSTONE_
/*******************************************************************************
*******************************************************************************/
OTX_RESULT OtxTsT1E1SetActiveMode(
  IN OTX_HANDLE hDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN TsT1E1ModeE eActiveMode);   // @parm mode Terminate/Monitor/Bridge/Loopback
#endif // _TURNSTONE_



/*******************************************************************************

  @func
    Increases the the receiver sensitivity to detect resistively
    attenuated signals of -20 dB. Typically used for in a monitor configuration.

  @comm
    This function is only valid for OTX_DEVICE_LI_T1E1 devices hosted by a 
    OTX_DEVICE_QUAD_CODEC device.

*******************************************************************************/
OTX_RESULT OtxT1E1SetMonitorMode(
  IN OTX_HANDLE hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN OTX_BOOL bMonitorModeActive  // @parm If OTX_TRUE the receiver sensitivity is increased to detect resistively attenuated signals of -20 dB
);

/*******************************************************************************

  @func
    Set the FS/DL-Bit data for the <e OtxLogicalDeviceTypeE.OTX_LDEVICE_T1_DL_BIT_ACCESS>
    device.

  @comm
    Start FS/DL bit access by calling function <f OtxDrvEnable>
    and end it with function <f OtxDrvDisable>

  @devnote
    Only meaningful in T1 mode and 

  @xref
    <e OtxLogicalDeviceTypeE.OTX_LDEVICE_T1_DL_BIT_ACCESS>
    <e OtxT1E1EventCodesE.OTX_T1E1_EC_DL_BITS_SENT>
    <e OtxT1E1EventCodesE.OTX_T1E1_EC_DL_BITS_RECEIVED>

*******************************************************************************/
OTX_RESULT OtxT1SetDlBitData(
  IN OTX_HANDLE hLogicDevice,        // @parm Handle to the T1 DL-Bit Access Logical Device
  IN OTX_UINT32 nDlBits              // @parm DL-Data to be transmitted.
                                     // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_F12>:
                                     // <nl> <p nDlBits> should contain 8 DL bits.
                                     // <nl> Bit0 is transmitted in frame 2 and Bit7 in frame 12.
                                     // <nl> Remaining bits should be zero.
                                     // <nl> 
                                     // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_ESF>:                                           
                                     // <nl> <p nDlBits> should contain 16 DL bits.
                                     // <nl> Bit0 is transmitted in frame  1 and Bit7  in frame 15.
                                     // <nl> Bit8 is transmitted in frame 17 and Bit15 in frame 23.
                                     // <nl> Remaining bits should be zero.
                                     // <nl> 
                                     // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_F72>:
                                     // <nl> <p nDlBits> should contain 24 DL bits.
                                     // <nl> Bit0  is transmitted in frame 26 and Bit7  in frame 40
                                     // <nl> Bit8  is transmitted in frame 42 and Bit15 in frame 56.
                                     // <nl> Bit16 is transmitted in frame 58 and Bit23 in frame 72.
                                     // <nl> Remaining bits should be zero.

);


/*******************************************************************************

  @func
    Set the Bit Rob Signalling information (all channels).

  @comm
    Start bit-rob signalling by calling function <f OtxDrvEnable>
    and end it with function <f OtxDrvDisable>

  @devnote
    Only meaningful in T1 mode and 
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_FALSE (which is the default)

  @xref
    <f OtxT1GetBitRobData>
    <nl><t OtxT1BitRobDataS>

*******************************************************************************/
OTX_RESULT OtxT1SetRobBitData(
  IN OTX_HANDLE hLogicDevice,        // @parm Handle to the T1 BitRob Access Logical Device
  IN OtxT1RobBitDataS *pRobBitData   // @parm Bit-robbed signalling data to be transmitted. See also <t OtxT1RobBitDataS>
);


/*******************************************************************************

  @func
    Get the current received bit rob signalling information (all channels).

  @devnote
    Only meaningful in T1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_FALSE (which is the default)

  @xref
    <f OtxT1SetBitRobData>
    <t OtxT1BitRobDataS>

*******************************************************************************/
OTX_RESULT OtxT1GetRobBitData(
  IN OTX_HANDLE hLogicDevice,           // @parm Handle to the T1 BitRob Access Logical Device
  OUT OtxT1RobBitDataS *pRobBitData     // @parm Bit-robbed signalling data received. See also <t OtxT1RobBitDataS>
);



/*******************************************************************************

  @func
    Set the Bit Rob Signalling information (ABCD bits) for a specific channel (1-24).

  @comm
    Start bit-rob signalling by calling function <f OtxDrvEnable>
    and end it with function <f OtxDrvDisable>

  @devnote
    Only meaningful in T1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_TRUE

  @xref
    <f OtxT1GetChannelizedBitRobData>
    <nl><e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>

*******************************************************************************/
OTX_RESULT OtxT1SetChannelizedRobBitData(
  IN OTX_HANDLE hLogicDevice,        // @parm Handle to the T1 S-Bit Access Logical Device
  IN OTX_INT32 nChannel,             // @parm Channel (1 - 24)
  IN OTX_UINT8 nABCDBits             // @parm The ABCD bits for this channel to be transmitted (A=bit3,B=bit2,C=bit1,D=bit0)
);


/*******************************************************************************

  @func
    Get the current received bit rob signalling (ABCD bits) for a specific channel (1-24).

  @devnote
    Only meaningful in T1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_TRUE

  @xref
    <f OtxT1SetChannelizedBitRobData>
    <nl><e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>

*******************************************************************************/
OTX_RESULT OtxT1GetChannelizedRobBitData(
  IN OTX_HANDLE hLogicDevice,           // @parm Handle to the T1 Bit Rob Access Logical Device
  IN OTX_INT32 nChannel,                // @parm Channel (1 - 24)
  OUT OTX_UINT8 *pnABCDBits             // @parm The ABCD bits for this channel are returned in the lower nibble of this parameter (A=bit3,B=bit2,C=bit1,D=bit0)
);



/*******************************************************************************

  @func
    Return the value of the received SaX bits. Returns 8-bits received
    during the last CRC-Multiframe.

  @comm
    Start signalling by calling function <f OtxDrvEnable> and end it
    with function <f OtxDrvDisable>

  @devnote
    Only meaningful in E1 mode

  @xref
    <f OtxE1SetSaBit>
    <nl><t OtxE1SaBitE>

*******************************************************************************/
OTX_RESULT OtxE1GetSaBit(
  IN OTX_HANDLE hLogicDevice, // @parm Handle to the E1 S-Bit Access Logical Device
  IN  OtxE1SaBitE eSaBit,     // @parm Sa bit to retrieve. See also <t OtxE1SaBitE>
  OUT OTX_UINT8 *pSaVal       // @parm Last 8 received Sa-bits
);



/*******************************************************************************

  @func
    Set the value to be sent at the SaX bits. Eight bits to be sent can be
    specified per Sa-bit.

  @comm
    In CRC-Multiframe format, one bit is sent in the corresponsing Sa-bit
    location of time-slot 0 in every other frame (in frames that do not
    contain frame alignment information). The least significant bit of the
    saVal byte is sent first in frame 1 of the multiframe and the most significant
    bit of the Byte is sent last in frame 15 of the multiframe.

    In Doubleframe format one bit of the saVal word is sent in the every other
    frame starting from the least significant bit.

    Start signalling by calling function <f OtxDrvEnable> and end it
    with function <f OtxDrvDisable>

  @devnote
    Only meaningful in E1 mode

  @xref
    <f OtxE1SaBitsEnable>
    <nl><f OtxE1SaBitsDisable>
    <nl><f OtxE1GetSaBit>
    <nl><t OtxE1SaBitE>


*******************************************************************************/
OTX_RESULT OtxE1SetSaBit(
  IN OTX_HANDLE hLogicDevice, // @parm Handle to the E1 S-Bit Access Logical Device
  IN OtxE1SaBitE eSaBit,      // @parm Sa bit to set. See also <t OtxE1SaBitE>
  IN OTX_UINT8 nSaVal         // @parm Byte (8 bits) to be sent. LSB will be sent first.
);



/*******************************************************************************

  @func
    Retrieve the value to Si bits received in the last frame.

  @comm
    In Doubleframe format, these are the first bits of each frame.

    In CRC-Multiframe format, the Si bit are the first bits of frames 13 and
    15. In CRC-Multiframe format these bits are also known as the E-bits.

  @devnote
    Only meaningful in E1 mode

  @xref
    <f OtxE1SetSiBit>

*******************************************************************************/
OTX_RESULT OtxE1GetSiBit(
  IN OTX_HANDLE hLogicDevice, // @parm Handle to the E1 S-Bit Access Logical Device
  OUT OTX_UINT8 *pSi1Val,     // @parm Value of the FAS Si-bit in doubleframe format
                              // or Si (E) bit in frame 13 in CRC-multiframe format
  OUT OTX_UINT8 *pSi2Val      // @parm Value of the service word Si bit in DoubleFrame format
                              // or Si (E) bit in frame 15 in CRC-multiframe format
);



/*******************************************************************************

  @func
    Set the value to be sent at the Si bit positions.

  @comm
    In Doubleframe format, these are the first bits of each frame.

    In CRC-Multiframe format, the Si bit are the first bits of frames 13 and
    15. In CRC-Multiframe format these bits are also known as the E-bits.

  @devnote
     Only meaningful in E1 mode

  @xref
    <f OtxE1GetSiBit>

*******************************************************************************/
OTX_RESULT OtxE1SetSiBit(
  IN OTX_HANDLE hLogicDevice, // @parm Handle to the E1 S-Bit Access Logical Device
  IN OTX_UINT8 nSi1Val,       // @parm Value of the FAS Si-bit in doubleframe format
                              // or Si (E) bit in frame 13 in CRC-multiframe format
  IN OTX_UINT8 nSi2Val        // @parm Value of the service word Si bit in DoubleFrame format
                              // or Si (E) bit in frame 15 in CRC-multiframe format
);



/*******************************************************************************

  @func
    Set the E1 ABCD Signalling information for Channel Associated
    Signalling (CAS)

  @comm
    Start signalling by calling function <f OtxDrvEnable> for the
    logical device and end it with function <f OtxDrvDisable>

  @devnote
    Only meaningful in E1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_FALSE (which is default)

  @xref
    <f OtxE1GetABCDBit>
    <nl><t OtxE1ABCDBitDataS>

*******************************************************************************/
OTX_RESULT OtxE1SetABCDBit(
  IN OTX_HANDLE hLogicDevice,        // @parm Handle to the E1 ABCD-Bit Access Logical Device
  IN OtxE1ABCDBitDataS *pABCDBitData // @parm ABCD CAS signalling data to be transmitted. See also <t OtxE1ABCDBitDataS>
);



/*******************************************************************************

  @func
    Get the latest received E1 ABCD Signalling information for
    Channel Associated Signalling (CAS)

  @comm
    Start signalling by calling function <f OtxDrvEnable> for the
    logical device and end it with function <f OtxDrvDisable>

  @devnote
    Only meaningful in E1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_FALSE (which is default)

  @xref
    <f OtxE1SetABCDBit>
    <nl><t OtxE1ABCDBitDataS>

*******************************************************************************/
OTX_RESULT OtxE1GetABCDBit(
  IN  OTX_HANDLE hLogicDevice,        // @parm Handle to the E1 ABCD-Bit Access Logical Device
  OUT OtxE1ABCDBitDataS *pABCDBitData // @parm ABCD CAS signalling data received. See also <t OtxE1ABCDBitDataS>
);



/*******************************************************************************

  @func
    Set the E1 ABCD Signalling information for a specific channel (1-30)
    for Channel Associated Signalling (CAS).

  @comm
    Start signalling by calling function <f OtxDrvEnable> for the
    logical device and end it with function <f OtxDrvDisable>

  @devnote
    Only meaningful in E1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_TRUE

  @xref
    <f OtxE1GetChannelizedABCDBit>
    <nl><t OtxE1ABCDBitDataS>

*******************************************************************************/
OTX_RESULT OtxE1SetChannelizedABCDBit(
  IN OTX_HANDLE hLogicDevice,        // @parm Handle to the E1 ABCD-Bit Access Logical Device
  IN OTX_INT32 nChannel,             // @parm CAS Channel (1 - 30)
  IN OTX_UINT8 nABCDBits             // @parm The ABCD bits for this channel to be transmitted (A=bit3,B=bit2,C=bit1,D=bit0)
);



/*******************************************************************************

  @func
    Get the latest received E1 ABCD Signalling information for a specific 
    channel (1-30) for Channel Associated Signalling (CAS).

  @devnote
    Only meaningful in E1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_TRUE

  @xref
    <f OtxE1SetChannelizedABCDBit>
    <nl><e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>

*******************************************************************************/
OTX_RESULT OtxE1GetChannelizedABCDBit(
  IN  OTX_HANDLE hLogicDevice,        // @parm Handle to the E1 ABCD-Bit Access Logical Device
  IN OTX_INT32 nChannel,              // @parm CAS Channel (1 - 30)
  OUT OTX_UINT8 *pnABCDBits           // @parm The ABCD bits for this channel are returned in the lower nibble of this parameter (A=bit3,B=bit2,C=bit1,D=bit0)
);

//***************************************************************************************
#ifdef _TURNSTONE_
//***************************************************************************************

OTX_RESULT OtxTsT1E1StartPrbsPattern(IN OTX_HANDLE hDevice);

OTX_RESULT OtxTsT1E1StopPrbsPattern(IN OTX_HANDLE hDevice);

OTX_RESULT OtxTsT1E1SetPrbsConfig(IN OTX_HANDLE hDevice,
					IN TsT1E1IoInPrbsConfigOptionsS *pPrbsConfigOptions);

OTX_RESULT OtxTsT1E1SetPrbsMonitor(IN OTX_HANDLE hDevice,
								   IN OTX_UINT8 nMonitorPrbs);

OTX_RESULT OtxTsT1E1ChangeT1Config(IN OTX_HANDLE hDevice, IN ConfigTypeE configType);


#ifndef _WIN32_WCE
OTX_RESULT OtxTsT1E1StartSecTimer(IN OTX_HANDLE hDevice,IN OTX_HANDLE hEvent);
#else //_WIN32_WCE
OTX_RESULT OtxTsT1E1StartSecTimer(IN OTX_HANDLE hDevice);
#endif //_WIN32_WCE


OTX_RESULT OtxTsT1E1EnableLiAutoDetection(IN OTX_HANDLE hDevice,
										  OtxT1FrameFormatE  ePreferredFF);

OTX_RESULT OtxTsT1E1EnablePulseAutoDetection(IN OTX_HANDLE hDevice);

OTX_RESULT OtxTsT1E1ErrorInsertion(IN OTX_HANDLE hDevice,
								   IN TsT1E1ErrorTypeE ErrorType);

OTX_RESULT OtxTsT1E1SetLBO(IN OTX_HANDLE hDevice,
						   IN TsT1E1LineBuildOutE eBuildOut);


#ifdef _TURNSTONE_TEST_
OTX_RESULT OtxTsT1E1TestSetRegister(IN OTX_HANDLE hDevice,
								OTX_UINT32 uRegAddress,OTX_UINT8 uSetValue,OTX_UINT8 uResetFalc);
OTX_RESULT OtxTsT1E1TestGetRegister(IN OTX_HANDLE hDevice,
								OTX_UINT32 uRegAddress,OTX_UINT8 *uGetValue);

OTX_RESULT OtxTsT1E1TestResetCounters(IN OTX_HANDLE hDevice,
								OTX_UINT32 uRegAddress);

OTX_RESULT OtxTsT1E1TestGetCounter(IN OTX_HANDLE hDevice,
								OTX_UINT32 uRegAddress,OTX_UINT32 *uRegCount);
#endif //_TURNSTONE_TEST_

//***************************************************************************************
#endif   //_TURNSTONE_
//***************************************************************************************


#ifdef __cplusplus
  } // extern "C"
#endif

/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998, 1999


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif   // OTX_T1E1_H
